﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace LinqDemo
{
    public partial class Form1 : Form
    {
        private IList<Person> m_Persons = null;

        public Form1()
        {
            InitializeComponent();
            InitializePeopleList();
        }

        private void InitializePeopleList()
        {
            #region C# 1.0/2.0 的集合初始化方式
            /*
            m_PeopleList = new List<Person>();

            Person person = new Person();
            person.FirstName = "Bob";
            person.LastName = "Li";
            person.Age = 25;
            m_PeopleList.Add(person);

            person = new Person();
            person.FirstName = "Eric";
            person.LastName = "Zhang";
            person.Age = 28;
            m_PeopleList.Add(person);

            person = new Person();
            person.FirstName = "David";
            person.LastName = "Liu";
            person.Age = 42;
            m_PeopleList.Add(person);
            */
            #endregion

            #region C# 3.0 的集合初始化方式
            // 集合初始化器(Collection Initializers)
            m_Persons = new List<Person>()
            {
                // 对象初始化器(Object Initializers)
                new Person { FirstName = "Bob", LastName = "Li", Age = 25 },
                new Person { FirstName = "Eric", LastName = "Zhang", Age = 28 },
                new Person { FirstName = "David", LastName = "Liu", Age = 42 }
            };
            #endregion
        }

        private void ShowResult(string title, IEnumerable<Person> persons)
        {
            txtResult.Text = title + ":" + Environment.NewLine;
            foreach (Person person in persons)
            {
                txtResult.Text = txtResult.Text + person.ToString() + Environment.NewLine;
            }
        }

        private void btnLambda_Click(object sender, EventArgs e)
        {
            // Lambda 表达式
            IEnumerable<Person> results = m_Persons.Where(p => p.Age > 25)
                                                                              .OrderBy(p => p.FirstName);
            ShowResult("Lambda", results);
        }

        private void btnQuerySyntax_Click(object sender, EventArgs e)
        {
            // 查询句法
            IEnumerable<Person> results = from s in m_Persons
                                         where s.Age > 25
                                         orderby s.FirstName descending
                                         select s;
            ShowResult("Query Syntax", results);
        }

        private void btnAnonymousTypes_Click(object sender, EventArgs e)
        {
             // 匿名类型
             var results = from s in m_Persons
                                         where s.Age < 42
                                         orderby s.FirstName
                                         select s;
             ShowResult("Anonymous Types", results);
        }
    }
}